#!/usr/bin/perl

=pod 

=head1 NAME

tp_fastacmd.pl 

=head1  SYNOPSIS 

tp_fastacmd.pl -i <tabulatted file> -o <fasta output file> [-l <int>] [-c <column>]


=head1 DESCRIPTION

    	Retrieves sequences

=head1 AUTHORS

    	Sebastien.Carrere@toulouse.inra.fr

=cut

BEGIN
{
		use FindBin;
        use lib "$FindBin::Bin/../../playmoby/lib";
}

use strict;
use ParamParser;
use General;
use Appli;
use Cwd 'abs_path';



my $FASTACMD_BIN = $FindBin::Bin . '/../ext/fastacmd_bin/fastacmd';
my $VERSION = `$FASTACMD_BIN - | grep ^fastacmd | cut -d ' ' -f2`;
chomp $VERSION;

my $DB_DIR     = '/db/generic';
my %H_DBS      = ('sp' => 'Swiss-Prot database', 'enzyme' => 'Enzyme database');
my $DEFAULT_DB = 'enzyme';
my %H_DEFAULT_VALUES = (column => 1, l => 80);

MAIN:
{
    my $o_param = New ParamParser('GETOPTLONG', ("o=s", "d=s", "i=s", "column=i", "l=i", "mobyle", "pmbtest", "debug"));
    
    &SetAppli(\$o_param);

    $o_param->AssertFileExists('i');
    $o_param->AssertDefined('o'); 
    $o_param->SetUnlessDefined('d', $DEFAULT_DB);
    $o_param->SetUnlessDefined('column', $H_DEFAULT_VALUES{column});
    $o_param->SetUnlessDefined('l', $H_DEFAULT_VALUES{l});
    
	$o_param->AssertAllowedValue('d', keys %H_DBS);
    
    
    my $fh_in = $o_param->GetStreamIn('i');
    my $accession_file = '/tmp/'. $$ . time;
    my $fh_out = General::GetStreamOut($accession_file);
    my $col_indice = $o_param->Get('column') - 1;
    my %h_accessions = ();
    while (my $line = <$fh_in>)
    {
		chomp $line;
		my @a_fields = split ("\t",$line);
		if (defined $a_fields[$col_indice]  && ($a_fields[$col_indice]  ne '') && ! (exists $h_accessions{$a_fields[$col_indice]}))
		{
			print $fh_out $a_fields[$col_indice] . "\n" ;
			$h_accessions{$a_fields[$col_indice]} = 1;
		}
	}	
	$fh_in->close;
	$fh_out->close;
	
    
    
    
    my $fastacmd_cmd =
      $FASTACMD_BIN 
      . ' -d ' . $DB_DIR . '/' . $o_param->Get('d') 
      . ' -i ' . $accession_file
      . ' -o ' . $o_param->Get('o') 
      . ' -l ' . $o_param->Get('l');
      
    print STDERR "INFO - $fastacmd_cmd\n" if ($o_param->IsDefined('debug'));
    my $ret = system ($fastacmd_cmd);
    print STDERR "ERROR -  $! \n" if ($ret < 0);
    unlink $accession_file if (! $o_param->IsDefined('debug'));
    
    
}

sub SetAppli
{
	my $ro_param = shift;
    my %h_general = (
                     name     => 'FastaCmd',
                     descr    => 'Retrieves sequences from Swiss-Prot or Enzyme',
                     category => 'Retrieval',
                     authors  => 'Sebastien.Carrere@toulouse.inra.fr',
                     version => $VERSION,
                     cmd      => abs_path($0)
                     );

    my %h_inputs =
      ('Infile' => {descr => 'Input tabulatted file', type_biomoby => 'text-formatted',  cmd => '-i $value'});
    my %h_outputs =
      ('MultiFasta' =>
        {descr => 'Multfasta output file', type_biomoby => 'FASTA_AA_multi', cmd => '-o $value'});

    my %h_parameters = (
                        'line_length' => {
                                           descr  => 'Sequence line length',
                                           type_biomoby => 'Integer',
                                           min     => 1,
                                           max	=> 99999999,
                                           default      => $H_DEFAULT_VALUES{l},
                                           cmd          => '-l $value'
                          },
                          'column' => {
                                           descr  => 'Column number in which to get accessions',
                                           type_biomoby => 'Integer',
                                           min => 1,
                                           max => 99999999,
                                           default      => $H_DEFAULT_VALUES{column},
                                           cmd          => '-column $value'
                          },
                          'database' => {
                                       descr  => 'Target database',
                                       type_biomoby => 'String',
                                       default      => $DEFAULT_DB,
                                       enum         => \%H_DBS,
                                       cmd          => '-d $value'
                          }
                          );

    my $o_appli = New Appli(-general => \%h_general);
    $o_appli->SetInputs(%h_inputs);
    $o_appli->SetOutputs(%h_outputs);
    $o_appli->SetParams(%h_parameters);
    $$ro_param->SetAppli($o_appli); 
    
    
    exit print STDOUT $o_appli->GetMobyleXml()       if ($$ro_param->IsDefined('mobyle'));
    exit print STDOUT $o_appli->GetPlaymobyTestXml() if ($$ro_param->IsDefined('pmbtest'));
    return ($o_appli);

}


