#!/usr/bin/perl

=pod 

=head1 NAME

tp_muscle.pl 

=head1  SYNOPSIS 

tp_muscle.pl -i <tabulatted file> -o <fasta output file> [-l <int>] [-c <column>]


=head1 DESCRIPTION

    	Retrieves sequences

=head1 AUTHORS

    	Sebastien.Carrere@toulouse.inra.fr

=cut

BEGIN
{
		use FindBin;
        use lib "$FindBin::Bin/../../playmoby/lib";
}

use strict;
use ParamParser;
use General;
use Appli;
use Cwd 'abs_path';



my $MUSCLE_BIN = $FindBin::Bin . '/../ext/muscle_bin/muscle -quiet -msf';
my $VERSION = `$MUSCLE_BIN -version`;
chomp $VERSION;

my %H_DEFAULT_VALUES = (diags => 0, maxiters => 16);

MAIN:
{
    my $o_param = New ParamParser('GETOPTLONG', ("out=s",  "in=s", "maxiters=i", "diags", "mobyle", "pmbtest", "debug"));
    
    &SetAppli(\$o_param);

    $o_param->AssertFileExists('in');
    $o_param->AssertDefined('out'); 
    $o_param->SetUnlessDefined('maxiters', $H_DEFAULT_VALUES{maxiters});
    
    
	
    
    
    
    my $muscle_cmd =
      $MUSCLE_BIN 
      . ' -in ' . $o_param->Get('in') 
      . ' -out ' . $o_param->Get('out') 
      . ' -maxiters ' . $o_param->Get('maxiters');
      
      $muscle_cmd .= ' -diags ' if ($o_param->IsDefined('diags'));
      
    print STDERR "INFO - $muscle_cmd\n" if ($o_param->IsDefined('debug'));
    my $ret = system ($muscle_cmd);
    print STDERR "ERROR -  $! \n" if ($ret < 0);
    
    
}

sub SetAppli
{
	my $ro_param = shift;
    my %h_general = (
                     name     => 'muscle',
                     descr    => 'Multiple alignment tool',
                     category => 'Multiple_Sequence_Comparison',
                     authors  => 'Sebastien.Carrere@toulouse.inra.fr',
                     version => $VERSION,
                     cmd      => abs_path($0)
                     );

    my %h_inputs =
      ('Multifasta' => {descr => 'Multfasta input file, nucleic or proteic', type_biomoby => 'FASTA',  cmd => '-in $value'});
    my %h_outputs =
      ('Alignment' =>
        {descr => 'Aligned sequences', type_biomoby => 'MSF', cmd => '-out $value'});

    my %h_parameters = (
                        'maxiters' => {
                                           descr  => 'Maximum number of iterations',
                                           type_biomoby => 'Integer',
                                           min     => 1,
                                           max 	   => 50,
                                           default      => $H_DEFAULT_VALUES{maxiters},
                                           cmd          => '-maxiters $value'
                          },
                          'diags' => {
                                           descr  => 'Find diagonals (faster for similar sequences)',
                                           type_biomoby => 'Boolean',
                                           default      => $H_DEFAULT_VALUES{diags},
                                           cmd          => '-diags'
                          }
                          );

    my $o_appli = New Appli(-general => \%h_general);
    $o_appli->SetInputs(%h_inputs);
    $o_appli->SetOutputs(%h_outputs);
    $o_appli->SetParams(%h_parameters);
    $$ro_param->SetAppli($o_appli); 
    
    
    exit print STDOUT $o_appli->GetMobyleXml()       if ($$ro_param->IsDefined('mobyle'));
    exit print STDOUT $o_appli->GetPlaymobyTestXml() if ($$ro_param->IsDefined('pmbtest'));
    return ($o_appli);

}


